/*
 * Copyright 2020 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package io.grpc.kotlin.generator.protoc

import com.google.protobuf.DescriptorProtos.FileDescriptorProto

/**
 * Describes a policy for converting proto message types to Java classes in the correct package.
 */
enum class JavaPackagePolicy {
  OPEN_SOURCE {
    override fun javaPackage(fileProto: FileDescriptorProto): PackageScope {
      return if (fileProto.options.hasJavaPackage()) {
        PackageScope(fileProto.options.javaPackage)
      } else {
        PackageScope(fileProto.`package`)
      }
    }
  };

  abstract fun javaPackage(fileProto: FileDescriptorProto): PackageScope
}
